//------------------------------------------------------------------------------
//
//  npglut.c
//
//  ANTz application based on NPE is free software
//	NPE is the Neural Physics Engine that drives the interactions
//
//	ANTz hosted at http://temporalzone.com and NPE at http://neuralphysics.org
//
//	Copyright (c) 2010-2011, Shane Saxon
//	All rights reserved.
//	Redistribution and use in source and binary forms, with or without 
//	modification, are permitted provided that the following conditions are met:
//		Redistributions of source code must retain the above copyright notice, 
//		this list of conditions and the following disclaimer.
//		Redistributions in binary form must reproduce the above copyright 
//		notice, this list of conditions and the following disclaimer in the 
//		documentation and/or other materials provided with the distribution.
//		Neither the name of the Shane Saxon nor the names of its contributors 
//		may be used to endorse or promote products derived from this software 
//		without specific prior written permission.
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
//	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
//	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//	CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
//	POSSIBILITY OF SUCH DAMAGE.
//
//------------------------------------------------------------------------------

#include "npglut.h"


#include "../npio.h"

#include "npgl.h"
#include "npkey.h"
#include "npmouse.h"

#include "../data/npmapfile.h"		//debug, zz

#define kWindowWidth 640
#define kWindowHeight 480
#define kWindowPositionX 50
#define kWindowPositionY 50


void npGlutInitGL();
void npGlutDrawGLScene(void);
void npGlutDrawGLSceneIdle(void);
void npGlutResizeGLScene(int Width, int Height);

void npGlutKeyDown (unsigned char key, int x, int y);
void npGlutKeyUp (unsigned char key, int x, int y);
void npGlutKeyDownSpecial (int key, int x, int y);
void npGlutKeyUpSpecial (int key, int x, int y);


//------------------------------------------------------------------------------
void npInitGlut (int argc, char **argv, void* dataRef)
{
	glutInit (&argc, argv);

	glutInitDisplayMode(GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE);

//	glutInitDisplayMode (GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);		//GLUT_RGB
	glutInitWindowPosition (kWindowPositionX, kWindowPositionY);
	glutInitWindowSize (kWindowWidth, kWindowHeight);
	glutCreateWindow ("ANTz");
	glutFullScreen ();

	//register keyboard events with GLUT
	glutKeyboardFunc (npGlutKeyDown);
	glutKeyboardUpFunc (npGlutKeyUp);
	glutSpecialFunc (npGlutKeyDownSpecial);
	glutSpecialUpFunc (npGlutKeyUpSpecial);
	
	//register mouse events with GLUT
	glutMouseFunc (npMouseEvent);
	glutMotionFunc (npMouseMotion);

	glutDisplayFunc (npGlutDrawGLScene);
	glutIdleFunc (npGlutDrawGLSceneIdle);

	glutReshapeFunc (npGLResizeScene);
	
	npInitGL (dataRef);

//	npSetMapFile  ("ANTZ0001.CSV", 1, 13, dataRef);						//move this debug, zz
}


//------------------------------------------------------------------------------
void npCloseGlut (void* dataRef)
{
	return;
}


//------------------------------------------------------------------------------
void npAppLoopGlut (void* dataRef)
{
	npInitGlut (0, 0, dataRef);
	glutMainLoop();
}


//------------------------------------------------------------------------------
void npGlutDrawGLScene(void) 
{
	static int previousTime = 0;
	int time = 0;
	int deltaTime = 0;

	npGLDrawScene (npGetDataRef());

	glutSwapBuffers();
	glutPostRedisplay();			//zz helps with forcing redraw, workaround, debug
									//issue with mouse hitting edge of screen
	while (deltaTime < 16)			//zz 1 = 1ms, 16 is about 60fps, improve this, debug
	{
	time = glutGet(GLUT_ELAPSED_TIME);
	deltaTime = time - previousTime;
	}
	//	printf("\n %d   %d", deltaTime, time);
	//	glfwSleep((double)0.1f);

	previousTime = time;
}

//------------------------------------------------------------------------------

void npGlutDrawGLSceneIdle(void)
{
	npGlutDrawGLScene();
	return;
}

//------------------------------------------------------------------------------
void npGlutFullScreenExit()
{
	glutPositionWindow (kWindowPositionX, kWindowPositionY);
	glutReshapeWindow (kWindowWidth, kWindowHeight);        /* Restore us */

	glutSwapBuffers();
	glutPostRedisplay();
}


//------------------------------------------------------------------------------
void npGlutKeyDown(unsigned char key, int x, int y) 
{
	npKeyGlut (key, x, y, kGlutKeyDown, glutGetModifiers());
}

//------------------------------------------------------------------------------
void npGlutKeyUp(unsigned char key, int x, int y) 
{
	npKeyGlut (key, x, y, kGlutKeyUp, glutGetModifiers());
}

//------------------------------------------------------------------------------
void npGlutKeyDownSpecial(int key, int x, int y) 
{
	npKeyGlut (key, x, y, kGlutKeyDownSpecial, glutGetModifiers());
}

//------------------------------------------------------------------------------
void npGlutKeyUpSpecial(int key, int x, int y) 
{
	npKeyGlut (key, x, y, kGlutKeyUpSpecial, glutGetModifiers() );
}


//------------------------------------------------------------------------------
void npGLSolidSphere (GLdouble radius, GLint slices, GLint stacks)
{
	glutSolidSphere (radius, slices, stacks);
}

//------------------------------------------------------------------------------
void npGLWireSphere (GLdouble radius, GLint slices, GLint stacks)
{
	glutWireSphere (radius, slices, stacks);
}

//------------------------------------------------------------------------------
void npGLSolidCone (GLdouble base, GLdouble height, GLint slices, GLint stacks)
{
	glutSolidCone (base, height, slices, stacks);
}

//------------------------------------------------------------------------------
void npGLWireCone (GLdouble base, GLdouble height, GLint slices, GLint stacks)
{
	glutWireCone (base, height, slices, stacks);
}

//------------------------------------------------------------------------------
void npGLSolidTorus (GLdouble innerRadius, GLdouble outerRadius, 
					GLint slices, GLint stacks)
{
	glutSolidTorus (innerRadius, outerRadius, slices, stacks);
}

//------------------------------------------------------------------------------
void npGLWireTorus (GLdouble innerRadius, GLdouble outerRadius, 
					GLint slices, GLint stacks)
{
	glutWireTorus (innerRadius, outerRadius, slices, stacks);
}

//------------------------------------------------------------------------------
void npGLPrimitive (int primitive)
{
	glPushMatrix();
	switch (primitive)
	{
	case kNPglutWireCube :
		glScalef (0.6f, 0.6f, 0.6f); 
		glutWireCube(2.0f);
		break;
	case kNPglutSolidCube :
		glScalef (0.6f, 0.6f, 0.6f);
		glutSolidCube(2.0f); 
		break;
	case kNPglutWireSphere : glutWireSphere( 1.0f, 15, 15 ); break;
	case kNPglutSolidSphere : glutSolidSphere( 1.0f, 15, 15 ); break;

	case kNPglutWireCone : glutWireCone( 1.0f, 2.0f, 20, 1 ); break;
	case kNPglutSolidCone : glutSolidCone( 1.0f, 2.0f, 20, 1 ); break;

	case kNPglutWireTorus : glutWireTorus(0.15f, 1.5f, 7, 16); break;
	case kNPglutSolidTorus : glutSolidTorus(0.15f, 1.5f, 7, 16); break;

	case kNPglutWireDodecahedron :
		glScalef (0.6f, 0.6f, 0.6f);
		glutWireDodecahedron(); 
		break;
	case kNPglutSolidDodecahedron :
		glScalef (0.6f, 0.6f, 0.6f);
		glutSolidDodecahedron();
		break;
	case kNPglutWireOctahedron : glutWireOctahedron(); break;
	case kNPglutSolidOctahedron : glutSolidOctahedron(); break;
	case kNPglutWireTetrahedron : glutWireTetrahedron(); break;
	case kNPglutSolidTetrahedron : glutSolidTetrahedron(); break;
	case kNPglutWireIcosahedron : glutWireIcosahedron(); break;
	case kNPglutSolidIcosahedron : glutSolidIcosahedron(); break;

	case kNPglutWireTeapot : glutWireTeapot( 2.0f ); break;
	case kNPglutSolidTeapot : glutSolidTeapot( 2.0f ); break;

	default : glutWireTetrahedron(); break;
	}
	glPopMatrix();
}