//------------------------------------------------------------------------------
//
//  npdata.c
//
//  ANTz application based on NPE is free software
//	NPE is the Neural Physics Engine that drives the interactions
//
//	ANTz hosted at http://temporalzone.com and NPE at http://neuralphysics.org
//
//	Copyright (c) 2010-2011, Shane Saxon
//	All rights reserved.
//	Redistribution and use in source and binary forms, with or without 
//	modification, are permitted provided that the following conditions are met:
//		Redistributions of source code must retain the above copyright notice, 
//		this list of conditions and the following disclaimer.
//		Redistributions in binary form must reproduce the above copyright 
//		notice, this list of conditions and the following disclaimer in the 
//		documentation and/or other materials provided with the distribution.
//		Neither the name of the Shane Saxon nor the names of its contributors 
//		may be used to endorse or promote products derived from this software 
//		without specific prior written permission.
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
//	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
//	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
//	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
//	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
//	CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
//	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
//	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
//	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
//	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
//	POSSIBILITY OF SUCH DAMAGE.
//
//------------------------------------------------------------------------------


#include "npdata.h"
#include "npctrl.h"
#include "data/npnode.h"
#include "data/npmap.h"
#include "io/npkey.h"
#include "io/npfile.h"
#include "io/npmouse.h"

#include <stdlib.h>

void* gData;

void npInitDataMap (void* dataRef);
void npInitDataIO (int argc, char** argv, void* dataRef);
void npInitDataCtrl (void* dataRef);


//------------------------------------------------------------------------------
void* npInitData (int argc, char** argv)
{
	// allocate memory for the global 'data' structure
	pData data = (pData) malloc (sizeof(Data));
	if (data == NULL)
	{
		printf ("\n 4213 error malloc failed cannot run app \n");
		return NULL;
	}

	// initialize data
	npInitDataMap (data);
	npInitDataIO (argc, argv, data);	//stores the command line arguements
	npInitDataCtrl (data);


	gData = data;						//store a copy for retrieval

	return gData;
	// create a fixed array of global data pointers that is thread safe, zz
	// need to support multiple instances within a single app
}


//------------------------------------------------------------------------------
void npInitDataMap(void* dataRef)
{
	int i = 0;
	pData data = (pData) dataRef;

	NPnodePtr node = NULL;
	NPnodePtr nodeChild = NULL;

	npInitMap (dataRef);
	npInitNode (dataRef);

	// global root reserved at index = 0
	npNodeNew (kNodeDefault, NULL, dataRef);

	// root camera, reserved at index = 1
	node = npNodeNew (kNodeCamera, NULL, dataRef);

	// child cameras attached to root camera
	for (i=0; i < kNPcameraCount; i++)
	{
		nodeChild = npNodeNew (kNodeCamera, node, dataRef);
		npDataCameraPreset (i, nodeChild, dataRef);
	}
	node->childIndex = 0;		// re-select the first camera

	// root grid at index = 2
	node = npNodeNew (kNodeGrid, NULL, dataRef);
	node->scale.z = 1.5f;

	// setup initial scene using preset 1
	//	npDataPreset (1, dataRef);

	// select the camera
	npCtrlCommand (kGraphCamera, dataRef);
}

//------------------------------------------------------------------------------
void npInitDataGL(void* dataRef)
{
	pData data = (pData) dataRef;
	
	data->io.gl.normal = kNPglNormalize;
	data->io.gl.shade = kNPglShadeSmooth; //kNPglShadeFlat; //works if starts Flat, debug zz
	
	data->io.gl.alphaMode = kNPalphaModeSubtractive;
}

//------------------------------------------------------------------------------
void npInitDataIO(int argc, char** argv, void* dataRef)
{
	int i = 0, j = 0;

	pData data = (pData) dataRef;

	// setup initial key command assignments
	npInitKey (dataRef);

	npInitMouse (dataRef);

	npInitDataGL (dataRef);

	// store reference to the command line arguements
	data->io.argc = argc;
	data->io.argv = argv;

	data->io.picking = false;

	data->io.drawMenu = false;

	data->io.clearFrameBuffer = true;
	data->io.clearIndex = 0;

	data->io.clear.r = 0.0f;
	data->io.clear.g = 0.0f;
	data->io.clear.b = 0.0f;
	data->io.clear.a = 0.0f;

	data->io.axes.x = true;
	data->io.axes.y = true;
	data->io.axes.z = true;


	for ( i=0; i < kNPchannelMax; i++)
		for ( j=0; j < kNPchannelSampleMax; j++)
			data->io.channel[i][j] = 0.0f;
	
	data->io.channelCount = 40;
	data->io.channelIndex = 0;
	data->io.channelOldest = 0;
	data->io.channelSize = 0;


	// clear the array
	for (i=0; i < kNPdataTypeMax; i++)
		data->io.write[i] = NULL;

	data->io.size = sizeof(NPio);
}


//------------------------------------------------------------------------------
void npInitDataCtrl(void* dataRef)
{
	pData data = (pData) dataRef;

	data->ctrl.slow = 0.2f;		//normalize speed to 1.0 and 5.0, debug zz
	data->ctrl.fast = 2.0f;		//do this in npengine

	data->ctrl.size = sizeof(NPctrl);
}


//------------------------------------------------------------------------------
void npCloseData()
{
	pData data = (pData) gData;
	
	free (data);
}


//------------------------------------------------------------------------------
void* npGetDataRef()
{
	return gData;
}


//------------------------------------------------------------------------------
void npUpdateData (void* dataRef)
{
	pData data = (pData) dataRef;

	//update the data->map

	return;
}

//------------------------------------------------------------------------------
char* npGetDataTypeName (int type)
{
	if (type > 0 )//&& type < kTypeCount)
		return NULL;//(data->map.typeList[type].name);
	else
		return NULL;
}
//------------------------------------------------------------------------------
char* npGetDataTypeDesc (int type)
{
	if (type > 0 )//&& type < kTypeCount)
		return NULL;//(data->map.typeList[type].description);
	else
		return NULL;
}


// loads preset scene using currently selected object as the origin
// if alt is pressed (ie ALT+5) it loads preset at the global origin
//------------------------------------------------------------------------------
void npDataPreset (int preset, void* dataRef)
{
	int i = 0;
	int j = 0;

	NPfloatXYZ	translateOffset;
	NPfloatXYZ	rotateOffset;
	NPfloatXYZ	scaleOffset;

	pData data = (pData) dataRef;
	NPnodePtr node = data->map.node[data->map.nodeRootIndex];

	NPnodePtr nodeChild = NULL;

	NPpinPtr pinData = NULL;

	// don't offset if alt key held or camera selected
	if (data->io.key.modAlt || data->map.nodeRootIndex == 1)
	{
		translateOffset.x = 0.0f;
		translateOffset.y = 0.0f;
		translateOffset.z = 0.0f;

		rotateOffset.x = 0.0f;
		rotateOffset.y = 0.0f;
		rotateOffset.z = 0.0f;

		scaleOffset.x = 0.0f;
		scaleOffset.y = 0.0f;
		scaleOffset.z = 0.0f;
	}
	else
	{
					// add code to calc translate, rotate position based on
					// global coordinates think of GL order of operation,	debug zz
		translateOffset.x = node->translate.x;
		translateOffset.y = node->translate.y;
		translateOffset.z = node->translate.z;

		rotateOffset.x = node->rotateRad.x;
		rotateOffset.y = node->rotateRad.y;
		rotateOffset.z = node->rotateRad.z;

		scaleOffset.x = node->scale.x;
		scaleOffset.y = node->scale.y;
		scaleOffset.z = node->scale.z;
	}

	switch (preset)
	{
		case kGraphPresetOne :
			node = npNodeNew (kNodePin, NULL, dataRef);
			node->colorIndex = 15;
			nodeChild = node->child[0];
				nodeChild->colorIndex = 14;

			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 1;
			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 14;
			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 1;
			node = npNodeNew (kNodePin, node, dataRef);
				node->colorIndex = 2;
			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 1;
			node = npNodeNew (kNodePin, node, dataRef);
				node->colorIndex = 3;
			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 4;

			npCtrlCommand (kGraphSelectAll, data);
			npCtrlCommand (kGraphColorUp, data);
			npCtrlCommand (kGraphColorDown, data);
			npCtrlCommand (kGraphSelectAll, data);

			data->map.nodeRootIndex = data->map.nodeRootCount - 1;
			data->map.selectedIndex = data->map.nodeRootIndex;
			data->map.selectedNode = data->map.node[data->map.selectedIndex];
			data->map.activeNode = data->map.selectedNode;

			break;
		case kGraphPresetTwo :
			// draw 6x6 grid of pins
			for(i = -3; i < 3; i++)
				for(j=-3; j < 3; j++)
				{
					// create the root pin
					node = npNodeNew (kNodePin, NULL, dataRef);
					node->translate.x = i * 10.0f;
					node->translate.y = j * 10.0f;

					node->translate.x += translateOffset.x;
					node->translate.y += translateOffset.y;
					node->translate.z += translateOffset.z;

						node->colorIndex = 15;
					nodeChild = node->child[0];
						nodeChild->colorIndex = 14;

					nodeChild = npNodeNew (kNodePin, node, dataRef);
						nodeChild->colorIndex = 1;
					nodeChild = npNodeNew (kNodePin, node, dataRef);
						nodeChild->colorIndex = 14;
					nodeChild = npNodeNew (kNodePin, node, dataRef);
						nodeChild->colorIndex = 1;
					node = npNodeNew (kNodePin, node, dataRef);
						node->colorIndex = 2;
					nodeChild = npNodeNew (kNodePin, node, dataRef);
						nodeChild->colorIndex = 1;
					node = npNodeNew (kNodePin, node, dataRef);
						node->colorIndex = 3;
					nodeChild = npNodeNew (kNodePin, node, dataRef);
						nodeChild->colorIndex = 4;
					data->map.nodeRootIndex = data->map.nodeRootCount - 1;
					data->map.selectedIndex = data->map.nodeRootIndex;
					data->map.selectedNode = data->map.node[data->map.selectedIndex];
					data->map.activeNode = data->map.selectedNode;
				}			
			break;
		case kGraphPresetThree :
			//first pin root
			node = npNodeNew (kNodePin, NULL, dataRef);
									node->rotateRate.z = 0.001f;
				node->colorIndex = 15;
			nodeChild = node->child[0];
				nodeChild->colorIndex = 14;
			//first pin children
			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 1;
			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 14;
			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 1;
			node = npNodeNew (kNodePin, node, dataRef);
				node->colorIndex = 2;
			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 1;
			node = npNodeNew (kNodePin, node, dataRef);
				node->colorIndex = 3;
			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 4;

			//next pin
			node = npNodeNew (kNodePin, NULL, dataRef);
									node->rotateRate.z = -0.001f;
				node->colorIndex = 15;
			nodeChild = node->child[0];
				nodeChild->colorIndex = 14;

			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 1;
			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 14;
			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 1;
			node = npNodeNew (kNodePin, node, dataRef);
				node->colorIndex = 2;
			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 1;
			node = npNodeNew (kNodePin, node, dataRef);
				node->colorIndex = 3;
			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 4;

			//next pin
			node = npNodeNew (kNodePin, NULL, dataRef);
				node->colorIndex = 15;
			nodeChild = node->child[0];
				nodeChild->colorIndex = 15;

			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 1;
			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 14;
			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 1;
			node = npNodeNew (kNodePin, node, dataRef);
								node->rotateRate.x = 0.001f;
				node->colorIndex = 2;
			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 1;
			node = npNodeNew (kNodePin, node, dataRef);
									node->rotateRate.x = 0.005f;
				node->colorIndex = 3;
			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 4;
			
			//next pin
			node = npNodeNew (kNodePin, NULL, dataRef);
									node->rotateRate.z = 0.005f;
				node->colorIndex = 15;
			nodeChild = node->child[0];
				nodeChild->colorIndex = 14;

			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 1;
			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 14;
			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 1;
			node = npNodeNew (kNodePin, node, dataRef);
				node->colorIndex = 2;
			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 2;
			node = npNodeNew (kNodePin, node, dataRef);
				node->colorIndex = 3;
			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 4;

			//next pin
			node = npNodeNew (kNodePin, NULL, dataRef);
									node->rotateRate.z = 0.001f;
				node->colorIndex = 15;
			nodeChild = node->child[0];
				nodeChild->colorIndex = 14;

			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 1;
			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 14;
			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 1;
			node = npNodeNew (kNodePin, node, dataRef);
				node->colorIndex = 2;
			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 1;
			node = npNodeNew (kNodePin, node, dataRef);
				node->colorIndex = 3;
			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 4;

			npCtrlCommand (kGraphSelectAll, data);
			npCtrlCommand (kGraphColorUp, data);
			npCtrlCommand (kGraphColorDown, data);
			npCtrlCommand (kGraphSelectAll, data);

				
			data->map.nodeRootIndex = data->map.nodeRootCount - 1;
			data->map.selectedIndex = data->map.nodeRootIndex;
			data->map.selectedNode = data->map.node[data->map.selectedIndex];
			data->map.activeNode = data->map.selectedNode;

			npCtrlCommand (kGraphNext, data);
			npCtrlCommand (kGraphNext, data);
			npCtrlCommand (kGraphNext, data);
			npCtrlCommand (kGraphNext, data);

			break;
		case kGraphPresetFour : 
			node = npNodeNew (kNodePin, NULL, dataRef);
				node->rotateRad.z = 1.7f;
				node->scale.x = 1.5f;
				node->scale.y = 1.5f;
				node->scale.z = 1.5f;

				node->translate.x -= 3.0f;
				node->translate.y -= 22.0f;

				node->colorIndex = 15;

			nodeChild = node->child[0];
				nodeChild->colorIndex = 14;
			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 15;
			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 15;
			node = npNodeNew (kNodePin, node, dataRef);
				node->colorIndex = 1;
			nodeChild = npNodeNew (kNodePin, node, dataRef);
				nodeChild->colorIndex = 14;
			node = npNodeNew (kNodePin, node, dataRef);
				node->colorIndex = 4;
			node = npNodeNew (kNodePin, node, dataRef);
				node->colorIndex = 3;
								node->rotateRad.x = -0.7f;
			node = npNodeNew (kNodePin, node, dataRef);
				node->colorIndex = 3;
								node->rotateRad.x = 0.7f;
			node = npNodeNew (kNodePin, node, dataRef);
				node->colorIndex = 4;
								node->rotateRad.x = 0.7f;
			node = npNodeNew (kNodePin, node, dataRef);
				node->colorIndex = 2;
								node->rotateRad.x = 0.7f;
			node = npNodeNew (kNodePin, node, dataRef);
				node->colorIndex = 1;
								node->rotateRad.x = 1.5f;


			npCtrlCommand (kGraphSelectAll, data);
			npCtrlCommand (kGraphColorUp, data);
			npCtrlCommand (kGraphColorDown, data);
			npCtrlCommand (kGraphSelectAll, data);

			data->map.nodeRootIndex = data->map.nodeRootCount - 1;
			data->map.selectedIndex = data->map.nodeRootIndex;
			data->map.selectedNode = data->map.node[data->map.selectedIndex];
			data->map.activeNode = data->map.selectedNode;
	
			break;
		case kGraphPresetFive : break;
		case kGraphPresetSix : break;
		case kGraphPresetSeven : break;
		case kGraphPresetEight : break;

		default : break;
	}

	npCtrlCommand (kGraphCamera, data);			// select the camera
}


//resets the camera node passed in, specific to the preset number
//------------------------------------------------------------------------------
void npDataCameraPreset (int preset, NPnodePtr node, void* dataRef)
{
	int id = 0;	
	int branchLevel = 0;

	NPnodePtr nodeParent = NULL;
	
	// retain parameters
	id = node->id;
	branchLevel = node->branchLevel;

	//reset to defaults
	npInitNodeDefault (node);				//update to retain certain values, debug zz
	npInitNodeCamera (node);

	node->id = id;
	node->type = kNodeCamera;
	node->branchLevel = branchLevel;

	//set preset values
	switch (preset)
	{
		case 0 :
			node->translate.x = -27.0f;
			node->translate.y = -25.0f;
			node->translate.z = 9.0f;
			
			node->rotate.angle = 0.0f;
			node->rotate.x = 0.0f;
			node->rotate.y = 1.0f;
			node->rotate.z = 0.0f;

			node->rotateRad.x = kPos2PI / 32.0f;
			node->rotateRad.z = kPos2PI / 12.0f;
			break;
		case 1 : 
			node->translate.x = -0.0f;
			node->translate.y = -0.0f;
			node->translate.z = 100.0f;
			
			node->rotate.angle = 0.0f;
			node->rotate.x = 0.0f;
			node->rotate.y = 1.0f;
			node->rotate.z = 0.0f;

			node->rotateRad.x = kPos2PI / 4.0f;
			break;
		case 2 : 
			node->translate.x = -0.0f;
			node->translate.y = -85.0f;
			node->translate.z = 7.0f;
			
			node->rotate.angle = 0.0f;
			node->rotate.x = 0.0f;
			node->rotate.y = 1.0f;
			node->rotate.z = 0.0f;
			break;
		case 3 : 
			node->translate.x = 85.0f;
			node->translate.y = 0.0f;
			node->translate.z = 7.0f;
			
			node->rotate.angle = 0.0f;
			node->rotate.x = 0.0f;
			node->rotate.y = 1.0f;
			node->rotate.z = 0.0f;

			node->rotateRad.z = -kPos2PI / 4.0f;		
			break;
		default : 
			node->translate.x = -27.0f;
			node->translate.y = -25.0f;
			node->translate.z = 9.0f;
			
			node->rotate.angle = 0.0f;
			node->rotate.x = 0.0f;
			node->rotate.y = 1.0f;
			node->rotate.z = 0.0f;

			node->rotateRad.x = kPos2PI / 32.0f;
			node->rotateRad.z = kPos2PI / 12.0f;
			//	node->triggerOnLow.z = true;		//prevents going below surface
			//	node->triggerLevelLow.z = 1.5f;
			break;
	}
}

